
# q-expansion of newform 2112.2.d.l, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2112
weight = 2
poly_data = [16, 0, -4, -4, 2, -2, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 2, 0, 0, 0, 0, 0, 0], 1], [[0, -4, 0, 2, 2, -1, 0, 1], 8], [[0, -12, 8, 6, 0, -3, 2, -1], 8], [[0, 4, 4, -2, 2, 1, 0, -1], 4], [[0, -12, -4, -2, 0, -1, 2, 1], 8], [[-24, -4, 0, 2, -4, 3, 2, 1], 8], [[16, 12, 16, -6, 0, -1, -2, -3], 8]]

hecke_ring_character_values = [[2047, [-1, 0, 0, 0, 0, 0, 0, 0]], [133, [1, 0, 0, 0, 0, 0, 0, 0]], [1409, [-1, 0, 0, 0, 0, 0, 0, 0]], [1729, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 1, 0, 0], [0, 0, -1, 0, 0, -1, -1, 0], [0, 0, 0, 0, 0, 0, 1, 0], [-1, 0, 0, 0, 0, 0, 0, 0], [1, 0, 0, -1, 0, 0, 0, 0], [0, 1, 0, 0, -1, 0, 0, 2], [0, -1, 0, 0, 1, 0, 0, 0], [-1, 1, -1, 1, 1, 1, 0, 0], [0, 0, 0, 0, 0, 0, 2, 0], [0, 0, -1, 0, 0, -1, -2, 0], [2, 1, -1, 2, 1, 1, 0, 0], [0, -1, -2, 0, 1, -2, 2, -2], [0, 0, 0, 0, 0, 0, 0, 2], [-3, -1, 0, -1, -1, 0, 0, 0], [0, 1, -2, 0, -1, -2, 1, 0], [-6, 0, -1, 2, 0, 1, 0, 0], [-3, 1, -2, 1, 1, 2, 0, 0], [0, -2, -3, 0, 2, -3, 0, -4], [-3, 1, -1, -1, 1, 1, 0, 0], [-4, 1, -4, 0, 1, 4, 0, 0], [0, 3, -2, 0, -3, -2, -3, 2], [-6, 0, -2, -2, 0, 2, 0, 0], [0, -1, -1, 0, 1, -1, 4, 0], [-4, -1, -1, 0, -1, 1, 0, 0], [0, -1, 0, 0, 1, 0, 0, 2], [0, -4, 0, 0, 4, 0, 2, -2], [0, -1, 4, -2, -1, -4, 0, 0], [-1, -1, 0, -1, -1, 0, 0, 0], [0, 3, -1, 0, -3, -1, -4, 0], [0, -2, -2, 0, 2, -2, -1, -4], [2, -1, 0, 0, -1, 0, 0, 0], [0, -1, -3, 0, 1, -3, -2, 0], [0, 3, -2, 0, -3, -2, -6, 0], [0, 1, -4, 0, -1, -4, -2, 2], [0, 3, 2, 0, -3, 2, -3, 2], [2, -3, 3, 2, -3, -3, 0, 0], [0, 2, 4, 0, -2, 4, 2, 2], [0, -1, -2, 2, -1, 2, 0, 0], [0, -2, 2, 0, 2, 2, -2, 0], [-2, 2, 1, 2, 2, -1, 0, 0], [4, -1, -3, 0, -1, 3, 0, 0], [-15, -1, -1, -1, -1, 1, 0, 0], [-2, 2, 2, 0, 2, -2, 0, 0], [0, 2, -6, 0, -2, -6, -8, -4], [0, 0, 4, 0, 0, 4, 8, 2], [0, 4, -6, 0, -4, -6, -2, 2], [0, 0, 1, 0, 0, 1, 4, 0], [-4, -1, 0, 2, -1, 0, 0, 0], [-2, -1, -1, 2, -1, 1, 0, 0], [0, 2, 0, 0, -2, 0, 6, 0], [-8, 0, 2, 0, 0, -2, 0, 0], [6, -2, 6, 0, -2, -6, 0, 0], [-4, -2, 5, 0, -2, -5, 0, 0], [0, -4, 4, 0, 4, 4, 2, -4], [-2, 3, -4, 0, 3, 4, 0, 0], [0, 3, 2, 0, -3, 2, 1, 4], [0, 0, 2, 0, 0, 2, -7, 4], [1, 3, -4, 1, 3, 4, 0, 0], [0, 2, -6, 0, -2, -6, -8, 0], [0, 0, 6, 0, 0, 6, 2, 2], [0, -5, 6, 0, 5, 6, 8, -2], [0, 1, -4, 0, -1, -4, -2, 0], [3, -3, 4, 1, -3, -4, 0, 0], [4, -1, 1, 0, -1, -1, 0, 0], [0, 2, 1, 0, -2, 1, -5, 0], [0, -2, -1, 0, 2, -1, 4, 0], [-4, 1, 2, 0, 1, -2, 0, 0], [-2, 3, -2, 4, 3, 2, 0, 0], [1, 4, 2, -1, 4, -2, 0, 0], [0, -1, 9, 0, 1, 9, -2, 0], [6, 3, -2, 0, 3, 2, 0, 0], [0, 2, 1, 0, -2, 1, 8, -4], [1, -2, 2, 3, -2, -2, 0, 0], [0, -4, 7, 0, 4, 7, 4, -4], [-3, 5, 3, -1, 5, -3, 0, 0], [0, 0, 7, 0, 0, 7, 5, 4], [-2, -2, 8, 0, -2, -8, 0, 0], [0, 0, 4, 0, 0, 4, 2, 4], [4, -2, -4, -2, -2, 4, 0, 0], [2, -4, -4, -2, -4, 4, 0, 0], [-8, 0, 4, -2, 0, -4, 0, 0], [2, -5, 0, -4, -5, 0, 0, 0], [0, 3, 3, 4, 3, -3, 0, 0], [0, 1, -8, 0, -1, -8, -7, -6], [-8, 2, -1, -4, 2, 1, 0, 0], [0, -3, -1, 0, 3, -1, 2, -8], [-4, -4, 2, 2, -4, -2, 0, 0], [0, 4, 4, 0, -4, 4, 0, 4], [0, -4, -7, 0, 4, -7, 0, 0], [10, -4, 7, 2, -4, -7, 0, 0], [-22, -1, -6, 4, -1, 6, 0, 0], [0, 2, 3, 0, -2, 3, -10, 4], [4, 2, 4, -4, 2, -4, 0, 0], [0, -2, -8, 0, 2, -8, -6, 2], [12, -2, 10, 0, -2, -10, 0, 0], [0, 0, 1, 0, 0, 1, -5, -4], [0, 1, -1, 0, -1, -1, 4, 4], [0, 1, -8, 0, -1, -8, 0, -4], [15, -2, 4, 1, -2, -4, 0, 0], [0, 2, 6, 0, -2, 6, 4, -6], [0, 5, 12, 0, -5, 12, 4, 6], [12, 0, -4, 0, 0, 4, 0, 0], [0, 2, -4, 0, -2, -4, -12, 4], [0, 2, -14, 0, -2, -14, -6, -2], [12, 3, 1, 0, 3, -1, 0, 0], [-16, -2, 4, 0, -2, -4, 0, 0], [0, -2, 0, 0, 2, 0, -8, 4], [1, -3, 4, -1, -3, -4, 0, 0], [0, 5, -2, 4, 5, 2, 0, 0], [0, -3, -6, 0, 3, -6, -5, 2], [-7, 1, 6, 5, 1, -6, 0, 0], [0, -2, 8, 0, 2, 8, 6, 8], [0, 0, -5, 0, 0, -5, -4, 4], [0, 4, -1, 0, -4, -1, -6, 8], [0, 5, 3, 0, -5, 3, 0, 4], [0, 8, -1, 0, -8, -1, -4, 4], [-1, -3, 9, 1, -3, -9, 0, 0], [0, 4, 1, 0, -4, 1, -13, 4], [8, 0, -2, 4, 0, 2, 0, 0], [6, 3, -5, 2, 3, 5, 0, 0], [12, 4, 4, 2, 4, -4, 0, 0], [0, -1, 0, 0, 1, 0, -4, -6], [2, 4, -8, 6, 4, 8, 0, 0], [0, -4, 3, 0, 4, 3, -4, -12], [0, -3, 0, 0, 3, 0, 12, -2], [26, -1, 7, -2, -1, -7, 0, 0], [21, 1, 5, 7, 1, -5, 0, 0], [0, 2, -17, 0, -2, -17, -18, -4], [-9, -7, 2, -5, -7, -2, 0, 0], [0, 0, -4, 0, 0, -4, 2, -2], [34, 4, -4, 2, 4, 4, 0, 0], [0, -2, -5, 0, 2, -5, 2, -4], [16, -2, -4, -6, -2, 4, 0, 0], [0, -5, 6, 0, 5, 6, -4, -6], [-6, 1, -12, 6, 1, 12, 0, 0], [0, 3, 10, 0, -3, 10, 1, 12], [0, 2, -14, 0, -2, -14, -16, -6], [0, -4, -3, 0, 4, -3, -1, 4], [0, 1, -14, 0, -1, -14, -14, -6], [0, 8, 4, 0, -8, 4, -16, 10], [0, -2, -2, 0, 2, -2, 8, -4], [0, -4, 5, 0, 4, 5, -4, -8], [-6, 3, -6, 0, 3, 6, 0, 0], [8, 5, -7, 0, 5, 7, 0, 0], [13, 1, 7, -1, 1, -7, 0, 0], [-13, -5, -4, 3, -5, 4, 0, 0], [0, 2, -4, 0, -2, -4, 2, 0], [0, 4, -5, 0, -4, -5, 4, 4], [9, 1, 4, -9, 1, -4, 0, 0], [-25, 2, -4, 1, 2, 4, 0, 0], [0, -1, -7, 0, 1, -7, -2, -16], [0, -8, 12, 0, 8, 12, 14, -2], [6, -6, -6, 2, -6, 6, 0, 0], [0, -2, -4, 0, 2, -4, -14, -6], [-15, -1, 4, 3, -1, -4, 0, 0], [0, 5, -2, 0, -5, -2, -1, 10], [0, -6, -8, 0, 6, -8, -4, -4], [6, -3, -4, 2, -3, 4, 0, 0], [0, -6, -10, 0, 6, -10, -4, -12], [4, 0, -3, -4, 0, 3, 0, 0], [0, -7, 8, 0, 7, 8, 2, -2], [0, -1, 8, 0, 1, 8, 15, 10], [12, 2, -3, 8, 2, 3, 0, 0], [0, -7, -3, 0, 7, -3, -4, 0], [1, -3, -5, -5, -3, 5, 0, 0], [0, 4, -4, 0, -4, -4, 4, -6], [-15, 0, -2, -1, 0, 2, 0, 0], [-16, 2, -8, 10, 2, 8, 0, 0], [0, 0, 7, 0, 0, 7, 21, 4], [-18, -4, 6, 2, -4, -6, 0, 0], [-14, 0, -12, 0, 0, 12, 0, 0], [30, 1, 2, -4, 1, -2, 0, 0], [10, 3, 8, -6, 3, -8, 0, 0], [0, 8, 1, 0, -8, 1, -4, 0], [0, -1, -11, 0, 1, -11, -12, 4], [0, 9, -6, 0, -9, -6, -2, 12], [0, 2, 7, 0, -2, 7, -7, 0], [0, -5, -8, 0, 5, -8, -5, -6], [17, 3, -6, 1, 3, 6, 0, 0], [0, 0, 20, 0, 0, 20, 16, 10], [-2, 0, -10, 2, 0, 10, 0, 0], [14, 1, 7, -2, 1, -7, 0, 0], [0, 6, 2, 0, -6, 2, 6, 4], [-1, -1, -9, -3, -1, 9, 0, 0], [0, -5, -14, 0, 5, -14, -1, 0], [15, 8, 0, -3, 8, 0, 0, 0], [0, 0, 12, 0, 0, 12, 18, 6], [-2, -5, -2, -10, -5, 2, 0, 0], [-6, 6, 6, 6, 6, -6, 0, 0], [0, -5, 1, -12, -5, -1, 0, 0], [10, -1, 20, 0, -1, -20, 0, 0], [0, 2, 1, 0, -2, 1, 0, 4], [0, -1, -6, 0, 1, -6, 5, 4], [0, -5, -2, -10, -5, 2, 0, 0], [0, 11, -7, 0, -11, -7, -10, 0], [8, -7, -2, -4, -7, 2, 0, 0], [-8, -1, -3, -4, -1, 3, 0, 0], [0, 6, -10, 0, -6, -10, -18, 4], [8, 5, -16, 6, 5, 16, 0, 0], [0, 5, -2, 0, -5, -2, -6, 6], [0, -1, -14, 0, 1, -14, -5, -10], [14, -4, -4, -4, -4, 4, 0, 0], [-4, 6, -10, 6, 6, 10, 0, 0], [24, -2, 11, 4, -2, -11, 0, 0], [0, -4, -9, 0, 4, -9, -17, 0], [0, -4, -11, 0, 4, -11, 0, -8], [14, 10, -12, 6, 10, 12, 0, 0], [0, 4, 4, 0, -4, 4, 12, 8], [0, 4, 8, 0, -4, 8, -10, 6], [2, 4, -10, 0, 4, 10, 0, 0], [0, -12, -5, 0, 12, -5, 15, -16], [0, -8, 7, 0, 8, 7, -2, 8], [-38, 4, -4, -2, 4, 4, 0, 0], [26, -5, -4, -4, -5, 4, 0, 0], [-4, 9, 5, 12, 9, -5, 0, 0], [0, -2, 2, 0, 2, 2, -1, -12], [0, -8, -16, 0, 8, -16, -4, -16], [9, 3, 8, -5, 3, -8, 0, 0], [0, 10, -5, 0, -10, -5, 7, 4], [-19, 2, 16, -1, 2, -16, 0, 0], [0, -2, 6, 0, 2, 6, 15, 4], [0, 9, -5, 0, -9, -5, 8, 12], [0, 2, -3, 0, -2, -3, -4, -4], [16, -7, 6, -6, -7, -6, 0, 0], [-13, 3, -16, 3, 3, 16, 0, 0], [0, -8, 8, 0, 8, 8, 12, 0], [7, -11, 8, -3, -11, -8, 0, 0], [0, 2, -18, 0, -2, -18, -11, 0], [-28, 3, -14, 2, 3, 14, 0, 0], [20, -3, 17, -4, -3, -17, 0, 0], [0, -3, -12, 0, 3, -12, 0, -12], [0, 3, 6, 0, -3, 6, 3, -10], [0, 10, 8, 0, -10, 8, 0, 4], [0, 16, -15, 0, -16, -15, -20, 0], [-4, 6, 2, 8, 6, -2, 0, 0], [-8, -1, -7, -4, -1, 7, 0, 0], [0, 3, 6, 0, -3, 6, -16, -2], [8, -8, -9, 0, -8, 9, 0, 0], [9, -5, -3, -1, -5, 3, 0, 0], [2, 4, -3, 2, 4, 3, 0, 0], [0, 8, -2, 0, -8, -2, -6, 2], [0, 2, 12, 0, -2, 12, -12, -2], [-18, 5, -13, -2, 5, 13, 0, 0], [0, -16, -4, 0, 16, -4, 16, -16], [-2, 2, -4, 2, 2, 4, 0, 0], [0, 2, -4, 0, -2, -4, -12, 6], [38, -6, 16, 6, -6, -16, 0, 0], [0, -2, 4, 0, 2, 4, 18, 2], [38, 0, -6, -2, 0, 6, 0, 0], [17, -7, 2, 5, -7, -2, 0, 0], [0, -3, 8, 0, 3, 8, 6, 6], [15, -1, 3, 5, -1, -3, 0, 0], [-2, 2, -4, -8, 2, 4, 0, 0], [0, 9, 0, 0, -9, 0, -2, 2], [26, -4, 0, -2, -4, 0, 0, 0], [6, -4, 12, 4, -4, -12, 0, 0], [0, 5, -10, 0, -5, -10, 6, 12], [0, -12, 15, 0, 12, 15, 7, -8], [-24, 3, 10, 0, 3, -10, 0, 0], [0, 11, 0, 0, -11, 0, -15, 6], [-18, -3, -2, 8, -3, 2, 0, 0], [-15, -6, 6, -5, -6, -6, 0, 0], [27, 8, -2, 1, 8, 2, 0, 0], [0, -12, 4, 0, 12, 4, -2, -12], [0, 0, 0, 0, 0, 0, 10, -2], [0, 4, -21, 0, -4, -21, -25, -8], [0, 3, 11, 0, -3, 11, 2, 4], [0, 6, 12, 0, -6, 12, 12, -2], [0, 0, 26, 0, 0, 26, 6, -4], [0, 7, 5, 4, 7, -5, 0, 0], [0, 2, 7, 0, -2, 7, -20, 8], [6, -2, 4, -8, -2, -4, 0, 0], [0, 4, -18, 0, -4, -18, -5, 0], [-22, 5, 0, -2, 5, 0, 0, 0], [0, -8, -5, 0, 8, -5, 6, 0], [-14, 10, -1, 6, 10, 1, 0, 0], [-11, -9, 8, 5, -9, -8, 0, 0], [-8, -1, 16, 0, -1, -16, 0, 0], [12, -5, -6, -14, -5, 6, 0, 0], [-2, -1, 10, 8, -1, -10, 0, 0], [0, 8, -5, 0, -8, -5, 6, 0], [-22, -1, 6, -4, -1, -6, 0, 0], [21, -3, 16, 1, -3, -16, 0, 0], [0, 8, 14, 0, -8, 14, 12, 14], [9, -3, -15, 3, -3, 15, 0, 0], [-18, 4, 4, 12, 4, -4, 0, 0], [0, -5, 0, 0, 5, 0, -2, 14], [0, 2, 7, 0, -2, 7, -2, 12], [0, -1, 12, 0, 1, 12, 16, 6], [0, -10, 15, 0, 10, 15, 13, 8], [-28, 2, -12, 4, 2, 12, 0, 0], [0, 4, 28, 0, -4, 28, 16, 0], [-12, -1, -4, 2, -1, 4, 0, 0], [-25, -6, 10, -7, -6, -10, 0, 0], [0, 17, -10, 0, -17, -10, -6, -2], [0, -10, 11, 0, 10, 11, 6, 4], [0, -6, 5, 0, 6, 5, -5, -16], [54, -1, 0, -4, -1, 0, 0, 0], [0, 2, -4, 0, -2, -4, 16, 6], [-2, 7, 2, 14, 7, -2, 0, 0], [0, 9, -2, 0, -9, -2, -12, 10], [0, 5, -8, 0, -5, -8, -11, 2]]
